#include "stdafx.h"
#include "HidDevice.h"
#pragma comment(lib, "SetupAPI.lib")
#pragma comment(lib, "hid.lib")


HidDevice::HidDevice()
  :mHandle(INVALID_HANDLE_VALUE)
{
}


HidDevice::~HidDevice()
{
  Close();
}

bool HidDevice::Open(uint16_t vendorId, uint16_t productId)
{

  if (mHandle != INVALID_HANDLE_VALUE) {
    return true;
  }

  GUID hidGuid; 
  HidD_GetHidGuid(&hidGuid);
 
  HDEVINFO devInfo = SetupDiGetClassDevs(&hidGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
  if (devInfo == 0) {
    return false;
  }

  SP_DEVICE_INTERFACE_DATA devData;
  devData.cbSize = sizeof(devData);
  for (DWORD index = 0; ; index++) {

    BOOL result = SetupDiEnumDeviceInterfaces(devInfo, NULL, &hidGuid, index, &devData);
    if (!result) {
      break;
    }

    DWORD length;
    SetupDiGetDeviceInterfaceDetail(devInfo, &devData, NULL, 0, &length, NULL);

    std::vector<BYTE> buf(length);
    PSP_DEVICE_INTERFACE_DETAIL_DATA devInfoDetail = reinterpret_cast<PSP_DEVICE_INTERFACE_DETAIL_DATA>(&buf[0]);
    devInfoDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
    result = SetupDiGetDeviceInterfaceDetail(devInfo, &devData, devInfoDetail, length, NULL, NULL);
    if (!result) {
      continue;
    }
    
    HANDLE devHandle = CreateFile(devInfoDetail->DevicePath,
      GENERIC_READ | GENERIC_WRITE,
      FILE_SHARE_READ | FILE_SHARE_WRITE,
      NULL, OPEN_EXISTING, 0, NULL);
    if (devHandle == INVALID_HANDLE_VALUE) {
      continue;
    }

    HIDD_ATTRIBUTES attr;
    attr.Size = sizeof(HIDD_ATTRIBUTES);
    HidD_GetAttributes(devHandle, &attr);
    if ((attr.VendorID == vendorId) && (attr.ProductID == productId)) {
      mHandle = devHandle;
      break;
    }
    CloseHandle(devHandle);
  }
  SetupDiDestroyDeviceInfoList(devInfo);

  return (mHandle != INVALID_HANDLE_VALUE);
}

void HidDevice::Close(void)
{
  if (mHandle != INVALID_HANDLE_VALUE) {
    CloseHandle(mHandle);
    mHandle = INVALID_HANDLE_VALUE;
  }
}

bool HidDevice::GetFeature(void *buf, int len)
{
  return HidD_GetFeature(mHandle, buf, len) != FALSE;

}

bool HidDevice::SetFeature(void* buf, int len)
{
  return HidD_SetFeature(mHandle, buf, len) != FALSE;
}

