#pragma once
#include "HidDevice.h"
#include "IRPoUsbData.h"

#define IR_DATA_NAME_MAX_LEN          6
#define IR_DATA_TIME_GROUP_MAX_COUNT  32
#define IR_DATA_TIMES_MAX_COUNT      1024
#pragma pack(push, 1)
typedef struct {
  char     name[IR_DATA_NAME_MAX_LEN + 1];
  uint8_t  checksum;
  uint16_t timesCount;
  uint8_t  angle;
  uint8_t  tilt;
  uint32_t alarmTime;
  uint8_t  times[IR_DATA_TIMES_MAX_COUNT];
  uint32_t timeGroup[IR_DATA_TIME_GROUP_MAX_COUNT];
} IR_DATA;
#pragma pack(pop)

class IRPoComError : public std::runtime_error {
public:
  explicit IRPoComError(const std::string &s)
       : std::runtime_error(s) {
       }
};

class IRPoCom
{
public:
  IRPoCom();
  ~IRPoCom();
  
  const uint16_t VENDOR_ID = 0x6666;
  const uint16_t DEVICE_ID = 0x2844;
  
  void Open();
  
  void SetTime(uint32_t epochTime);
  uint32_t GetTime();
  
  bool IsStudyDataAvailable();
  void GetCurrentPos(int *angle, int *tilt);
  uint8_t GetRomUpdateStatus();
  
  bool GetStudyData(IR_DATA *studyData);
  void WriteIRData(const IR_DATA *irData);
  void IRPoCom::ClearStudyData(void);
  
protected:

  void GetStatus(IRPO_USB_STATUS &status);
  
  void SendToDevice(int reqId, const void *data, int len);
  void RecvFromDevice(int reqId, void *data, int len);

  void SetFeatureWrapper(const void* data);
  void GetFeatureWrapper(void *data);

  HidDevice mHid;
};

