#include "IRData.h"
#include <chip.h>

static bool PrepareWriteFlash(uint32_t sectorNo)
{
  uint32_t cmd[5], result[4];

  cmd[0] = IAP_PREWRRITE_CMD;
  cmd[1] = sectorNo;
  cmd[2] = sectorNo;
  iap_entry(cmd, result);
  if (result[0] != IAP_CMD_SUCCESS) {
    return false;
  }
  return true;
}

static bool ErasePage(uint32_t pageNo, uint32_t pageCount)
{
  uint32_t cmd[5], result[4];

  cmd[0] = IAP_ERASE_PAGE_CMD;
  cmd[1] = pageNo;
  cmd[2] = pageNo + pageCount - 1;
  cmd[3] = SystemCoreClock / 1000;
  iap_entry(cmd, result);
  if (result[0] != IAP_CMD_SUCCESS) {
    return false;
  }
  return true;
}

static bool CopyToFlash(uint32_t flashAddr, const void* data, int size)
{
  uint32_t cmd[5], result[4];

  cmd[0] = IAP_WRISECTOR_CMD;
  cmd[1] = flashAddr;
  cmd[2] = (uint32_t)data;
  cmd[3] = size;
  cmd[4] = SystemCoreClock / 1000;

  iap_entry(cmd, result);
  if (result[0] != IAP_CMD_SUCCESS) {
    return false;
  }
  return true;
}

static bool WriteFlash(uint32_t flashAddr, const void* data, int size)
{
  uint32_t sectorNo, pageNo, pageCount;

  if (flashAddr >= 0x00018000) {
    sectorNo = ((flashAddr - 0x00018000) / 0x8000) + 24;
  } else {
    sectorNo = flashAddr / 0x1000;
  }

  if (! PrepareWriteFlash(sectorNo)) {
    return false;
  }

  pageNo = flashAddr / 256;
  pageCount = size / 256;
  if (! ErasePage(pageNo, pageCount)) {
    return false;
  }

  if (! PrepareWriteFlash(sectorNo)) {
    return false;
  }
  return CopyToFlash(flashAddr, data, size);
}

bool IRDataWriteToFlash(int index, const IR_DATA* data)
{
  bool writeResult = false;
  const uint8_t *src = (const uint8_t*)data;
  uint32_t flashAddr = (uint32_t)IRDataGetPointer(index);

  __disable_irq();

  writeResult = WriteFlash(flashAddr, src, 1024);
  if (! writeResult) {
    goto func_end;
  }
  writeResult = WriteFlash(flashAddr + 1024, src + 1024, 256);

func_end:
  __enable_irq();
  return writeResult;
}

uint8_t IRDataCalcChecksum(const IR_DATA *data)
{
  int i;
  uint8_t checksum = 0;
  const uint8_t *p = (const uint8_t*)data;
  for (i = 0; i < sizeof(IR_DATA); i++) {
    checksum += *p++;
  }
  return checksum;
}

