#ifndef IRDATA_H_
#define IRDATA_H_
#include <stdint.h>
#include <stdbool.h>

#define IR_DATA_NAME_MAX_LEN          6
#define IR_DATA_TIME_GROUP_MAX_COUNT  32
#define IR_DATA_TIMES_MAX_COUNT      1024

#pragma pack(push, 1)
typedef struct {
  char     name[IR_DATA_NAME_MAX_LEN + 1];
  uint8_t  checksum;
  uint16_t timesCount;
  uint8_t  angle;
  uint8_t  tilt;
  uint32_t alarmTime;
  uint8_t  times[IR_DATA_TIMES_MAX_COUNT];
  uint32_t timeGroup[IR_DATA_TIME_GROUP_MAX_COUNT];
} IR_DATA __attribute__((aligned(2)));
#pragma pack(pop)

#define IR_DATA_FLASH_BASE   0x00018000
#define IR_DATA_ENTRY_SIZE   1280
#define IR_DATA_ALARM_INDEX  (8 * 3)

static inline int AngleTiltToIndex(int angle, int tilt)
{
  return (angle * 3) + tilt;
}

static inline const IR_DATA* IRDataGetPointer(int index)
{
  return (const IR_DATA*)(IR_DATA_FLASH_BASE + IR_DATA_ENTRY_SIZE * index);
}

bool IRDataWriteToFlash(int index, const IR_DATA* data);
uint8_t IRDataCalcChecksum(const IR_DATA *data);

#endif
