#include "IRPoDisplay.h"
#include "mono_bitmap.h"
#include "AQM1248A.h"
#include "IRData.h"
#include <time.h>
#include <stdio.h>

extern const MONO_BITMAP_FONT gFont11x11;
extern const MONO_BITMAP_FONT gFont5x7;

static uint8_t mDispayRam[sizeof(MONO_BITMAP) + LCD_MEMORY_SIZE];
static const char* mStatusString = NULL;

static volatile int mDispAngle = 0;
static volatile int mDispTilt = 0;

typedef struct {
  uint8_t x;
  uint8_t y;
} POINT_XY;

static const POINT_XY mPointData[8][3] = {
  {{24,  3}, {24, 10}, {24, 17}}, // angle 0
  {{39,  9}, {34, 14}, {29, 19}}, // angle 1
  {{45, 24}, {38, 24}, {31, 24}}, // angle 2
  {{39, 39}, {34, 34}, {29, 29}}, // agnle 3
  {{24, 45}, {24, 38}, {24, 31}}, // angle 4
  {{ 9, 39}, {14, 34}, {19, 29}}, // angle 5
  {{ 3, 24}, {10, 24}, {17, 24}}, // angle 6
  {{ 9,  9}, {14, 14}, {19, 19}}  // angle 7
};

static void DrawChart(MONO_BITMAP *bitmap, int angle, int tilt)
{
  const POINT_XY *pointXY;

  MonoBitmapDrawCircle(bitmap, 24, 24, 23);
  MonoBitmapDrawLine(bitmap, 24, 0, 24, 23 * 2);
  MonoBitmapDrawLine(bitmap, 0, 24, 23 * 2, 24);

  pointXY = &mPointData[angle][tilt];
  MonoBitmapFillRect(bitmap, pointXY->x - 2, pointXY->y - 2, pointXY->x + 2, pointXY->y + 2);
}

static void PrintDeviceName(MONO_BITMAP *bitmap, int angle, int tilt)
{
  const IR_DATA *data;
  
  MonoBitmapSetFont(bitmap, &gFont11x11);

  data = IRDataGetPointer(AngleTiltToIndex(angle, tilt));
  if ((data->timesCount == 0) || (data->timesCount > IR_DATA_TIMES_MAX_COUNT)) {
    // データなし
    MonoBitmapPrintString(bitmap, 49, 2,  "NO");
    MonoBitmapPrintString(bitmap, 49, 2 + 11,  "DATA");
  } else if (*(data->name) == '\0') {
    // 名前なし
    char str[7];
    MonoBitmapPrintString(bitmap, 49, 2,  "NONAME");
    
    sprintf(str, "A:%dT:%d", angle, tilt);
    MonoBitmapPrintString(bitmap, 49, 2 + 11, str);
  } else {
    // 名前あり
    char str[7];
    MonoBitmapPrintString(bitmap, 49, 2,  data->name);
    
    sprintf(str, "A:%dT:%d", angle, tilt);
    MonoBitmapPrintString(bitmap, 49, 2 + 11, str);
  }
}

static void PrintTime(MONO_BITMAP *bitmap)
{
  // "123456789ABCD"
  //   12/31 12:34
  char str[12];
  time_t t;
  struct tm *tmp;
  
  t = time(NULL);
  tmp = localtime(&t);
  strftime(str, sizeof(str), "%m/%d %H:%M", tmp);
  
  MonoBitmapSetFont(bitmap, &gFont5x7);
  MonoBitmapPrintString(bitmap, 49 + 5, 2 + 11 * 2 + 3, str);
}

static void PrintStatus(MONO_BITMAP *bitmap)
{
  if (mStatusString != NULL) {
    MonoBitmapSetFont(bitmap, &gFont5x7);
    MonoBitmapPrintString(bitmap, 49 + 7, 2 + 11 * 2 + 3 + 8, mStatusString);
  }
}

static void Update(void)
{
  MONO_BITMAP *bitmap = (MONO_BITMAP*)mDispayRam;
  
  MonoBitmapClear(bitmap, MONO_BITMAP_WHITE);
  
  DrawChart(bitmap, mDispAngle, mDispTilt);
  PrintDeviceName(bitmap, mDispAngle, mDispTilt);
  PrintTime(bitmap);
  PrintStatus(bitmap);
  
  LCDSyncMemory(bitmap->data);

}



void IRPoDisplayUpdate(int angle, int tilt)
{
  mDispAngle = angle;
  mDispTilt = tilt;
  Update();
}

void IRPoDisplayUsbMode(void)
{
  MONO_BITMAP *bitmap = (MONO_BITMAP*)mDispayRam;
  
  MonoBitmapClear(bitmap, MONO_BITMAP_WHITE);
  
  MonoBitmapSetFont(bitmap, &gFont11x11);
  MonoBitmapPrintString(bitmap, 18, 18, "USB MODE");
  
  LCDSyncMemory(bitmap->data);
}

void IRPoDisplaySetStatusString(const char* str)
{
  mStatusString = str;
  Update();
}

void IRPoDisplayInitialize(void)
{
  
  MONO_BITMAP *bitmap = (MONO_BITMAP*)mDispayRam;
  
  LCDInitialize();

  MonoBitmapSetSize(bitmap, LCD_WIDTH, LCD_HEIGHT);
  MonoBitmapSetColor(bitmap, MONO_BITMAP_BLACK);

  IRPoDisplayUpdate(0, 0);
}

void IRPoDisplayOff(void)
{
  LCDEnterSleepMode();
}

void IRPoDisplayOn(void)
{
  LCDExitSleepMode();
}

