#include "i2c.h"
#include <chip.h>

#define I2C_FREQ   200000

static I2CM_XFER_T  i2cmXferRec;

static bool SetupXferRecAndExecute(uint8_t devAddr,
                   uint8_t *txBuffPtr,
                   uint16_t txSize,
                   uint8_t *rxBuffPtr,
                   uint16_t rxSize)
{
  /* Setup I2C transfer record */
  i2cmXferRec.slaveAddr = devAddr;
  i2cmXferRec.options = 0;
  i2cmXferRec.status = 0;
  i2cmXferRec.txSz = txSize;
  i2cmXferRec.rxSz = rxSize;
  i2cmXferRec.txBuff = txBuffPtr;
  i2cmXferRec.rxBuff = rxBuffPtr;
  if (Chip_I2CM_XferBlocking(LPC_I2C0, &i2cmXferRec)) {
    return true;
  }
  return false;
}

bool I2CRegWriteByte(uint8_t devAddr, uint8_t regAddr, uint8_t data)
{
  uint8_t txBuffer[2];
  txBuffer[0] = regAddr;
  txBuffer[1] = data;
  return SetupXferRecAndExecute(devAddr, txBuffer, 2, NULL, 0);
}

uint8_t I2CRegReadByte(uint8_t devAddr, uint8_t regAddr)
{
  uint8_t txBuffer[1];
  uint8_t rxBuffer[1];
  txBuffer[0] = regAddr;
  rxBuffer[0] = 0;
  if (SetupXferRecAndExecute(devAddr, txBuffer, 1, rxBuffer, 1)) {
    return rxBuffer[0];
  }
  return 0;
}

bool I2CRegReadMulti(uint8_t devAddr, uint8_t regAddr, uint8_t *buf, uint8_t size)
{
  uint8_t txBuffer[1];
  txBuffer[0] = regAddr;
  if (SetupXferRecAndExecute(devAddr, txBuffer, 1, buf, size)) {
    return true;
  }
  return false;
}

void InitializeI2C(void)
{
  Chip_I2CM_Init(LPC_I2C0);
  Chip_I2CM_SetBusSpeed(LPC_I2C0, I2C_FREQ);
  
  NVIC_DisableIRQ(I2C0_IRQn);


}



