/*
  以下を改変
  https://developer.mbed.org/users/masato/code/aqm1248a_lcd/
*/
/*
 * mbed library for the Akizuki AQM1248A 128*48 pixel LCD
 * derived from C12832_lcd
 * Copyright (c) 2014 Masato YAMANISHI
 * Released under the MIT License: http://mbed.org/license/mit
 */

/* mbed library for the mbed Lab Board  128*32 pixel LCD
 * use C12832 controller
 * Copyright (c) 2012 Peter Drescher - DC2PD
 * Released under the MIT License: http://mbed.org/license/mit
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include "mono_bitmap.h"
#include <string.h>

int MonoBitmapGetDataSize(const MONO_BITMAP *bitmap)
{
  return bitmap->width * bitmap->height / 8;
}

void MonoBitmapSetSize(MONO_BITMAP *bitmap, int width, int height)
{
  bitmap->width = width;
  bitmap->height = height;
}

void MonoBitmapSetFont(MONO_BITMAP *bitmap, const MONO_BITMAP_FONT *font)
{
  bitmap->font = font;
}

void MonoBitmapSetColor(MONO_BITMAP *bitmap, MONO_BITMAP_COLOR color)
{
  bitmap->color = color;
}

void MonoBitmapClear(MONO_BITMAP *bitmap, MONO_BITMAP_COLOR color)
{
  int size = MonoBitmapGetDataSize(bitmap);
  if (color) {
    memset(bitmap->data, 0xFF, size);
  } else {
    memset(bitmap->data, 0x00, size);
  }
}

void MonoBitmapSetPixel(MONO_BITMAP *bitmap, int x, int y)
{
  int byteIndex;
  int bitPos;

  if(x > bitmap->width || y > bitmap->height || x < 0 || y < 0) {
    return;
  }
  
  byteIndex = x + (y / 8) * bitmap->width;
  bitPos = y & 0x07;
  
  if (bitmap->color) {
    bitmap->data[byteIndex] |= (1 << bitPos);
  } else {
    bitmap->data[byteIndex] &= ~(1 << bitPos);
  }
}

void MonoBitmapDrawLine(MONO_BITMAP *bitmap, int x0, int y0, int x1, int y1)
{
  int   dx = 0, dy = 0;
  int   dx_sym = 0, dy_sym = 0;
  int   dx_x2 = 0, dy_x2 = 0;
  int   di = 0;

  dx = x1-x0;
  dy = y1-y0;

  if (dx > 0) {
    dx_sym = 1;
  } else {
    dx_sym = -1;
  }
  
  if (dy > 0) {
    dy_sym = 1;
  } else {
    dy_sym = -1;
  }

  dx = dx_sym*dx;
  dy = dy_sym*dy;

  dx_x2 = dx*2;
  dy_x2 = dy*2;

  if (dx >= dy) {
    di = dy_x2 - dx;
    while (x0 != x1) {

      MonoBitmapSetPixel(bitmap, x0, y0);
      x0 += dx_sym;
      if (di<0) {
        di += dy_x2;
      } else {
        di += dy_x2 - dx_x2;
        y0 += dy_sym;
      }
    }
    MonoBitmapSetPixel(bitmap, x0, y0);
  } else {
    di = dx_x2 - dy;
    while (y0 != y1) {
      MonoBitmapSetPixel(bitmap, x0, y0);
      y0 += dy_sym;
      if (di < 0) {
        di += dx_x2;
      } else {
        di += dx_x2 - dy_x2;
        x0 += dx_sym;
      }
    }
    MonoBitmapSetPixel(bitmap, x0, y0);
  }
}

void MonoBitmapDrawRect(MONO_BITMAP *bitmap, int x0, int y0, int x1, int y1)
{
  
  if (x1 > x0) {
    MonoBitmapDrawLine(bitmap, x0,y0,x1,y0);
  } else {
    MonoBitmapDrawLine(bitmap, x1,y0,x0,y0);
  }
  
  if (y1 > y0) {
    MonoBitmapDrawLine(bitmap, x0,y0,x0,y1);
  } else {
    MonoBitmapDrawLine(bitmap, x0,y1,x0,y0);
  }
  
  if (x1 > x0) {
    MonoBitmapDrawLine(bitmap, x0,y1,x1,y1);
  } else {
    MonoBitmapDrawLine(bitmap, x1,y1,x0,y1);
  }
  
  if (y1 > y0) {
    MonoBitmapDrawLine(bitmap, x1,y0,x1,y1);
  }  else {
    MonoBitmapDrawLine(bitmap, x1,y1,x1,y0);
  }
}

void MonoBitmapFillRect(MONO_BITMAP *bitmap, int x0, int y0, int x1, int y1)
{
  int l,c,i;
  
  if(x0 > x1) {
    i = x0;
    x0 = x1;
    x1 = i;
  }

  if(y0 > y1) {
    i = y0;
    y0 = y1;
    y1 = i;
  }
  
  for(l = x0; l<= x1; l ++) {
    for(c = y0; c<= y1; c++) {
      MonoBitmapSetPixel(bitmap, l, c);
    }
  }
}

void MonoBitmapDrawCircle(MONO_BITMAP *bitmap, int x0, int y0, int r)
{

  int draw_x0, draw_y0;
  int draw_x1, draw_y1;
  int draw_x2, draw_y2;
  int draw_x3, draw_y3;
  int draw_x4, draw_y4;
  int draw_x5, draw_y5;
  int draw_x6, draw_y6;
  int draw_x7, draw_y7;
  int xx, yy;
  int di;
  
  if (r == 0) {       /* no radius */
    return;
  }
  
  draw_x0 = draw_x1 = x0;
  draw_y0 = draw_y1 = y0 + r;
  if (draw_y0 < bitmap->height) {
    MonoBitmapSetPixel(bitmap, draw_x0, draw_y0);     /* 90 degree */
  }

  draw_x2 = draw_x3 = x0;
  draw_y2 = draw_y3 = y0 - r;
  if (draw_y2 >= 0) {
    MonoBitmapSetPixel(bitmap, draw_x2, draw_y2);    /* 270 degree */
  }

  draw_x4 = draw_x6 = x0 + r;
  draw_y4 = draw_y6 = y0;
  if (draw_x4 < bitmap->width) {
    MonoBitmapSetPixel(bitmap, draw_x4, draw_y4);     /* 0 degree */
  }

  draw_x5 = draw_x7 = x0 - r;
  draw_y5 = draw_y7 = y0;
  if (draw_x5>=0) {
    MonoBitmapSetPixel(bitmap, draw_x5, draw_y5);     /* 180 degree */
  }

  if (r == 1) {
    return;
  }
  
  di = 3 - 2*r;
  xx = 0;
  yy = r;
  while (xx < yy) {

    if (di < 0) {
      di += 4*xx + 6;
    } else {
      di += 4*(xx - yy) + 10;
      yy--;
      draw_y0--;
      draw_y1--;
      draw_y2++;
      draw_y3++;
      draw_x4--;
      draw_x5++;
      draw_x6--;
      draw_x7++;
    }
    xx++;
    draw_x0++;
    draw_x1--;
    draw_x2++;
    draw_x3--;
    draw_y4++;
    draw_y5++;
    draw_y6--;
    draw_y7--;

    if ( (draw_x0 <= bitmap->width) && (draw_y0>=0) ) {
      MonoBitmapSetPixel(bitmap, draw_x0, draw_y0);
    }

    if ( (draw_x1 >= 0) && (draw_y1 >= 0) ) {
      MonoBitmapSetPixel(bitmap, draw_x1, draw_y1);
    }

    if ( (draw_x2 <= bitmap->width) && (draw_y2 <= bitmap->height) ) {
      MonoBitmapSetPixel(bitmap, draw_x2, draw_y2);
    }

    if ( (draw_x3 >=0 ) && (draw_y3 <= bitmap->height) ) {
      MonoBitmapSetPixel(bitmap, draw_x3, draw_y3);
    }

    if ( (draw_x4 <= bitmap->width) && (draw_y4 >= 0) ) {
      MonoBitmapSetPixel(bitmap, draw_x4, draw_y4);
    }

    if ( (draw_x5 >= 0) && (draw_y5 >= 0) ) {
      MonoBitmapSetPixel(bitmap, draw_x5, draw_y5);
    }
    if ( (draw_x6 <=bitmap->width) && (draw_y6 <= bitmap->height) ) {
      MonoBitmapSetPixel(bitmap, draw_x6, draw_y6);
    }
    if ( (draw_x7 >= 0) && (draw_y7 <= bitmap->height) ) {
      MonoBitmapSetPixel(bitmap, draw_x7, draw_y7);
    }
  }
}

void MonoBitmapFillCircle(MONO_BITMAP *bitmap, int x0, int y0, int r)
{
  int i;
  for (i = 0; i <= r; i++) {
    MonoBitmapDrawCircle(bitmap, x0, y0, i);
  }
}

int MonoBitmapPutChar(MONO_BITMAP *bitmap, int x, int y, char c)
{
  const MONO_BITMAP_FONT *font;
  const uint8_t *fontOfChar;
  int widthOfChar, heightOfChar;
  int bytePerX;
  int xInChar, yInChar;

  if (bitmap->font == NULL) {
    return 0;
  }
  
  font = bitmap->font;
  if ((c < font->startChar) || (c >= font->endChar)) {
    return 0;
  }
  
  fontOfChar = &font->data[(c - font->startChar) * font->bytePerChar];
  widthOfChar = fontOfChar[0];
  fontOfChar++;
  
  heightOfChar = font->height;
  bytePerX = font->bytePerX;
  
  for (yInChar = 0; yInChar < heightOfChar; yInChar++) {
    for (xInChar = 0; xInChar < widthOfChar; xInChar++) {
      uint8_t d = fontOfChar[(xInChar * bytePerX) + ((yInChar & ~0x07) >> 3)];
      uint8_t m = 1 << (yInChar & 0x07);
      if (d & m) {
        MonoBitmapSetPixel(bitmap, x + xInChar, y + yInChar);
      }
    }
  }
  
  return widthOfChar;
}

void MonoBitmapPrintString(MONO_BITMAP *bitmap, int x, int y, const char* str)
{
  char c;
  int charWidth;
  int currentX = x;
  while ((c = *str++) != 0) {
    charWidth = MonoBitmapPutChar(bitmap, currentX, y, c);
    currentX += (charWidth + 1);
  }
}
