#include "rtc.h"
#include <chip.h>
#include <time.h>

void RTC_IRQHandler(void)
{
  // バグっている ので Chip_RTC_ClearStatus(LPC_RTC, RTC_CTRL_ALARM1HZ); は使わず直接
  LPC_RTC->CTRL |= RTC_CTRL_ALARM1HZ;
}

unsigned int __sys_time(void)
{
  return Chip_RTC_GetCount(LPC_RTC);
}

void RTCSetTimeByEpoch(uint32_t epochTime)
{
  Chip_RTC_Disable(LPC_RTC);
  Chip_RTC_SetCount(LPC_RTC, epochTime);
  Chip_RTC_Enable(LPC_RTC);
}

void RTCSetTimeByTM(struct tm *tmStruct)
{
  RTCSetTimeByEpoch((uint32_t)mktime(tmStruct));
}

bool RTCIsTimeValid(void)
{
  struct tm *pTM;
  time_t currentTime = time(NULL);
  pTM = localtime(&currentTime);
  if ((pTM->tm_year + 1900) < 2015) {
    return false;
  }
  return true;
}

void RTCSetAlarmByEpoch(uint32_t epochTime)
{
  Chip_RTC_SetAlarm(LPC_RTC, epochTime);
}

void RTCSetAlarmByTM(struct tm *tmStruct)
{
  RTCSetAlarmByEpoch((uint32_t)mktime(tmStruct));
}

void RTCInitialize(bool *alarmed)
{
  bool reseted;
  uint32_t backupCount;
  
  Chip_Clock_EnableRTCOsc();
  Chip_RTC_Init(LPC_RTC);

  if (LPC_RTC->CTRL & (RTC_CTRL_SWRESET | RTC_CTRL_OFD)) {
    reseted = true;
  } else {
    reseted = false;
  }
  
  if (alarmed != NULL) {
    if (LPC_RTC->CTRL & RTC_CTRL_ALARM1HZ) {
      *alarmed = true;
    } else {
      *alarmed = false;
    }
  }
  
  backupCount = Chip_RTC_GetCount(LPC_RTC);
  
  Chip_RTC_Reset(LPC_RTC);

  if (reseted) {
    struct tm tmStruct = {0};
    tmStruct.tm_year = 2015 - 1900;
    tmStruct.tm_mon = 11;
    tmStruct.tm_mday = 30;
    tmStruct.tm_hour = 17;
    tmStruct.tm_min = 36;
    tmStruct.tm_sec = 0;
    RTCSetTimeByTM(&tmStruct);
  } else {
    Chip_RTC_SetCount(LPC_RTC, backupCount);
  }
  
  Chip_RTC_Enable(LPC_RTC);
  
  Chip_RTC_SetAlarm(LPC_RTC, RTC_INVALID_ALARM_TIME);
  LPC_RTC->CTRL |= (RTC_CTRL_OFD | RTC_CTRL_ALARM1HZ | RTC_CTRL_WAKE1KHZ); // 1WriteClear
  NVIC_EnableIRQ(RTC_IRQn);
  Chip_SYSCTL_EnablePeriphWakeup(SYSCTL_WAKEUP_RTCINT);
  Chip_RTC_EnableWakeup(LPC_RTC, RTC_CTRL_ALARMDPD_EN);
}

