/*
 * @brief Common SystemInit function for LPC11u6x chips
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013-14
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "chip.h"

const uint32_t OscRateIn = 12000000;
const uint32_t RTCOscRateIn = 32768;

#define USE_LPCEXPRESSO_BOARD 0

static const uint32_t PORT0_OUTPIN_BITMAP = 0x745980;
static const uint32_t PORT1_OUTPIN_BITMAP = 0xFFFFDFFF;
static const uint32_t PORT2_OUTPIN_BITMAP = 0xFFFFFC;



static const PINMUX_GRP_T pinArray[] = {
  {0,  0, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // Reset Pullup無 (外でPullupしてある)
  {0,  1, (IOCON_FUNC0 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN)},  // ISP_N
  {0,  2, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // SSP0_SSEL (LCD_CS_N)
  {0,  3, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // VBUS
  {0,  4, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // I2C SCL
  {0,  5, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // I2C SDA
#if USE_LPCEXPRESSO_BOARD
  {0,  6, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO(----)
#else
  {0,  6, (IOCON_FUNC2 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // SSP0_SCK
#endif
  {0,  7, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (IR_POWER)
  {0,  8, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (----)
  {0,  9, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // SSP0_MOSI
  {0, 10, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // SWCLK
  {0, 11, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (PIO0_11 / ADC9として利用することも可)
  {0, 12, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (----)
  {0, 13, (IOCON_FUNC3 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // CT32B1_MAT0 (LED)
  {0, 14, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (----)
  {0, 15, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // SWDIO
  {0, 16, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (Wakup / INT1)
  {0, 17, (IOCON_FUNC2 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // CT32B0_CAP0 (IR_IN)
  {0, 18, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO (----)
  {0, 19, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // U0_TXD(UART_TX)
  {0, 20, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO(RXD2  : U2_RXDとして利用することも可)
  {0, 21, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO(MOSI2 : SSP1_MOSIとして利用することも可)
  {0, 22, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)},  // GPIO(MISO2 : SSP1_MISOとして利用することも可)
  {0, 23, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_ADMODE_EN)},   // ADC1 (VBAT)

  {1,  0, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  1, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  2, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  3, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  4, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  5, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  6, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  7, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  8, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1,  9, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 10, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 11, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 12, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 13, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (INT2)
  {1, 14, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 15, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 16, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 17, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 18, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 19, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 20, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (SCK2  : SSP1_SCKとして利用することも可) 
  {1, 21, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (MOSI2 : SSP1_MOSIとして利用することも可)
  {1, 22, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 23, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (TXD2  : U2_TXDとして利用することも可)
  {1, 24, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (PIO1_24)
  {1, 25, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 26, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 27, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 28, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
#if USE_LPCEXPRESSO_BOARD
  {1, 29, (IOCON_FUNC1 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // SSP0_SCK
#else
  {1, 29, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
#endif
  {1, 30, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {1, 31, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)

  {2,  0, (IOCON_FUNC1 | IOCON_MODE_INACT)}, // XTALIN
  {2,  1, (IOCON_FUNC1 | IOCON_MODE_INACT)}, // XTALOUT
  {2,  2, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (LCD_RS)
  {2,  3, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2,  4, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2,  5, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2,  6, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2,  7, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (PIO2_7)
  {2,  8, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2,  9, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 10, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 12, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 12, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 13, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 14, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 15, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 16, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 17, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 18, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 19, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 20, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 22, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 22, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  {2, 23, (IOCON_FUNC0 | IOCON_MODE_INACT  | IOCON_DIGMODE_EN)}, // GPIO (----)
  
};

void IRPOSetupMuxing(void)
{
  Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
  Chip_IOCON_SetPinMuxing(LPC_IOCON, pinArray, sizeof(pinArray) / sizeof(PINMUX_GRP_T));
}

void IRPoSetupClocking(void)
{
  Chip_SetupXtalClocking();
}

void IRPoSetupGpio(void)
{
  Chip_GPIO_Init(LPC_GPIO);

  Chip_GPIO_SetPortValue(LPC_GPIO, 0, 0);
  Chip_GPIO_SetPortValue(LPC_GPIO, 1, 0);
  Chip_GPIO_SetPortValue(LPC_GPIO, 2, 0);

  Chip_GPIO_SetPortDIROutput(LPC_GPIO, 0, PORT0_OUTPIN_BITMAP);
  Chip_GPIO_SetPortDIROutput(LPC_GPIO, 1, PORT1_OUTPIN_BITMAP);
  Chip_GPIO_SetPortDIROutput(LPC_GPIO, 2, PORT2_OUTPIN_BITMAP);
}

void IRPoBoradSystemInit(void)
{
  IRPOSetupMuxing();
  IRPoSetupClocking();
  IRPoSetupGpio();
}

/* Set up and initialize hardware prior to call to main */
void SystemInit(void)
{
  IRPoBoradSystemInit();
}
